#include "BlockType.h"

#include "BasicBlocks.h"
#include "Block.h"
#include "Game.h"
#include "ItemType.h"
#include "MultiblockStructure.h"

using namespace Framework;

BlockType::BlockType(Block* defaultBlock,
    ModelInfo* model,
    bool needsClientInstance,
    int initialMaxHP,
    bool lightSource,
    Framework::Text name,
    bool needModelSubscription,
    int initialMapColor,
    Framework::RCArray<Framework::Text> groupNames,
    float hardness)
    : ReferenceCounter(),
      id(id),
      model(model),
      initialMaxHP(initialMaxHP),
      needsClientInstance(needsClientInstance),
      lightSource(lightSource),
      name(name),
      needModelSubscription(needModelSubscription),
      initialMapColor(initialMapColor),
      defaultBlock(defaultBlock),
      groupNames(groupNames),
      hardness(hardness)
{}

BlockType::~BlockType()
{
    if (defaultBlock) defaultBlock->release();
}

void BlockType::loadSuperBlock(
    Block* zBlock, Framework::StreamReader* zReader, int dimensionId) const
{
    zBlock->loadInventory(zReader);
    zReader->lese((char*)&zBlock->transparent, 1);
    zReader->lese((char*)&zBlock->passable, 1);
    zReader->lese((char*)&zBlock->hp, 4);
    zReader->lese((char*)&zBlock->maxHP, 4);
    zReader->lese((char*)&zBlock->hardness, 4);
    zReader->lese((char*)&zBlock->speedModifier, 4);
    zReader->lese((char*)&zBlock->mapColor, 4);
    zReader->lese((char*)&zBlock->interactable, 1);
    int strCount;
    zReader->lese((char*)&strCount, 4);
    for (int i = 0; i < strCount; i++)
    {
        __int64 id;
        zReader->lese((char*)&id, 8);
        MultiblockStructure* str
            = Game::INSTANCE->zDimension(dimensionId)->zStructureById(id);
        if (str)
        {
            zBlock->structures.add(
                dynamic_cast<MultiblockStructure*>(str->getThis()));
        }
    }
}

void BlockType::saveSuperBlock(
    Block* zBlock, Framework::StreamWriter* zWriter) const
{
    zBlock->saveInventory(zWriter);
    zWriter->schreibe((char*)&zBlock->transparent, 1);
    zWriter->schreibe((char*)&zBlock->passable, 1);
    zWriter->schreibe((char*)&zBlock->hp, 4);
    zWriter->schreibe((char*)&zBlock->maxHP, 4);
    zWriter->schreibe((char*)&zBlock->hardness, 4);
    zWriter->schreibe((char*)&zBlock->speedModifier, 4);
    zWriter->schreibe((char*)&zBlock->mapColor, 4);
    zWriter->schreibe((char*)&zBlock->interactable, 1);
    int strCount = zBlock->structures.getEintragAnzahl();
    zWriter->schreibe((char*)&strCount, 4);
    for (MultiblockStructure* structure : zBlock->structures)
    {
        __int64 id = structure->getStructureId();
        zWriter->schreibe((char*)&id, 8);
    }
}

void BlockType::createSuperBlock(Block* zBlock, Item* zItem) const
{
    if (zItem)
    {
        BasicBlockItem* item = dynamic_cast<BasicBlockItem*>(zItem);
        if (item)
        {
            zBlock->transparent = item->transparent;
            zBlock->passable = item->passable;
            zBlock->hardness = item->hardness;
            zBlock->speedModifier = item->speedModifier;
            zBlock->interactable = item->interactable;
        }
    }
    zBlock->mapColor = initialMapColor;
}

void BlockType::createSuperItem(Block* zBlock, Item* zItem) const
{
    BasicBlockItem* item = dynamic_cast<BasicBlockItem*>(zItem);
    if (item)
    {
        item->transparent = zBlock->transparent;
        item->passable = zBlock->passable;
        item->hardness = zBlock->hardness;
        item->speedModifier = zBlock->speedModifier;
        item->interactable = zBlock->interactable;
    }
}

bool BlockType::initialize(Game* zGame)
{
    return true;
}

Framework::Text BlockType::getTargetUIML() const
{
    return Text("<targetInfo><text width=\"auto\" height=\"auto\">") + name
         + "</text></targetInfo>";
}

Block* BlockType::loadBlock(Framework::Vec3<int> position,
    Framework::StreamReader* zReader,
    int dimensionId) const
{
    Block* result = createBlock(position, dimensionId);
    loadSuperBlock(result, zReader, dimensionId);
    return result;
}

void BlockType::saveBlock(Block* zBlock, Framework::StreamWriter* zWriter) const
{
    saveSuperBlock(zBlock, zWriter);
}

Item* BlockType::getItemFromBlock(Block* zBlock) const
{
    Item* result = createItem();
    if (result)
    {
        createSuperItem(zBlock, result);
    }
    return result;
}

Block* BlockType::createBlockAt(
    Framework::Vec3<int> position, int dimensionId, Item* zUsedItem) const
{
    Block* result = createBlock(position, dimensionId);
    createSuperBlock(result, zUsedItem);
    return result;
}

bool BlockType::doesNeedClientInstance() const
{
    return needsClientInstance;
}

ModelInfo* BlockType::zModel() const
{
    return model;
}

int BlockType::getMapColor() const
{
    return initialMapColor;
}

const Framework::RCArray<Framework::Text>& BlockType::getGroupNames() const
{
    return groupNames;
}

float BlockType::getHardness() const
{
    return hardness;
}

int BlockType::getId() const
{
    return id;
}

const Block* BlockType::zDefault() const
{
    return defaultBlock;
}

int BlockType::getInitialMaxHP() const
{
    return initialMaxHP;
}

bool BlockType::isLightSource() const
{
    return lightSource;
}

const Block* getDefaultBlock(Either<Block*, int> b)
{
    if (b.isA())
        return b;
    else
        return Game::INSTANCE->zBlockType(b)->zDefault();
}

BlockType* BlockType::initializeDefault()
{
    if (!defaultBlock)
    {
        defaultBlock = createBlockAt({0, 0, 0}, 0, 0);
    }
    return this;
}

const char* BlockType::getName() const
{
    return name;
}

const bool BlockType::doesNeedModelSubscription() const
{
    return needModelSubscription;
}

void BlockType::writeTypeInfo(StreamWriter* zWriter) const
{
    int id = getId();
    zWriter->schreibe((char*)&id, 4);
    bool inst = doesNeedClientInstance();
    zWriter->schreibe((char*)&inst, 1);
    bool sub = doesNeedModelSubscription();
    zWriter->schreibe((char*)&sub, 1);
    bool fluid = isFluid();
    zWriter->schreibe((char*)&fluid, 1);
    if (fluid)
    {
        char flowDist = getFlowDistance();
        zWriter->schreibe(&flowDist, 1);
    }
    int maxHp = getInitialMaxHP();
    zWriter->schreibe((char*)&maxHp, 4);
    zModel()->writeTo(zWriter);
}

bool BlockType::isFluid() const
{
    return false;
}

int BlockType::getTypeId(const char* name)
{
    Text n = name;
    for (int i = 0; i < Game::INSTANCE->getBlockTypeCount(); i++)
    {
        if (Game::INSTANCE->zBlockType(i)
            && n.istGleich(Game::INSTANCE->zBlockType(i)->getName()))
            return Game::INSTANCE->zBlockType(i)->getId();
    }
    return 0;
}

Framework::Text BlockType::getTypeName(int id)
{
    for (int i = 0; i < Game::INSTANCE->getBlockTypeCount(); i++)
    {
        if (Game::INSTANCE->zBlockType(i)
            && Game::INSTANCE->zBlockType(i)->getId() == id)
            return Game::INSTANCE->zBlockType(i)->getName();
    }
    return 0;
}

unsigned char BlockType::getFlowDistance() const
{
    return 0;
}

void BlockType::setTypeId(int id)
{
    this->id = id;
}
