#pragma once

#include "Block.h"

class GrowingPlantBlockType;

struct GrowthState : public Framework::ReferenceCounter
{
private:
    float percentage;
    ModelInfo* model;

public:
    GrowthState(float percentage, ModelInfo* model);
    ~GrowthState();

    float getPercentage() const;
    ModelInfo* zModel() const;
};

class GrowingPlantBlock : public Block
{
private:
    float seblingTicks;
    int seblingTicksMax;
    Framework::Text name;
    Framework::RCArray<GrowthState> states;
    int blockTypeAfterGrowth;
    bool plantSpawned;
    int lastSendState;

public:
    GrowingPlantBlock(int typeId,
        Framework::Vec3<int> pos,
        int dimensionId,
        int maxTicks,
        Framework::Text name,
        int blockTypeAfterGrowth);
    virtual bool onTick(
        TickQueue* zQueue, int numTicks, bool& blocked) override;
    virtual void onPostTick() override;
    virtual void sendModelInfo(NetworkMessage* zMessage) override;
    virtual Framework::Text getTargetUIML();
    GrowingPlantBlock* addGrowthState(GrowthState* state);

    friend GrowingPlantBlockType;
};

class GrowingPlantBlockType : public BlockType
{
private:
    bool transparent;
    bool passable;
    float speedModifier;
    bool interactable;
    Framework::RCArray<GrowthState> states;
    int blockTypeIdAfterGrowth;
    Framework::Text blockTypeNameAfterGrowth;
    const char* readableName;
    int ticksNeeded;

public:
    GrowingPlantBlockType(ModelInfo* model,
        Framework::Text name,
        Framework::Text blockTypeNameAfterGrowth,
        const char* readableName,
        int ticksNeeded,
        int mapColor,
        float hardness,
        Framework::RCArray<Framework::Text> groupNames);

protected:
    virtual void loadSuperBlock(Block* zBlock,
        Framework::StreamReader* zReader,
        int dimensionId) const override;
    void createSuperBlock(Block* zBlock, Item* zItem) const override;
    virtual void saveSuperBlock(
        Block* zBlock, Framework::StreamWriter* zWriter) const override;
    virtual Item* createItem() const override;
    virtual Block* createBlock(
        Framework::Vec3<int> position, int dimensionId) const override;

public:
    GrowingPlantBlockType* addGrowthState(
        float growthPercentage, ModelInfo* model);

    Framework::Text getBlockTypeNameAfterGrowth() const;
    const char* getReadableName() const;
    int getTicksNeeded() const;
    const Framework::RCArray<GrowthState>& getStates() const;
    virtual ItemType* createItemType() const override;
};

class GrowingPlantBlockTypeFactory
    : public SubTypeFactory<BlockType, GrowingPlantBlockType>
{
public:
    GrowingPlantBlockTypeFactory();
    GrowingPlantBlockType* fromJson(
        Framework::JSON::JSONObject* zJson) const override;
    Framework::JSON::JSONObject* toJson(
        GrowingPlantBlockType* zObject) const override;
    Framework::JSON::Validator::JSONValidator* getValidator(
        Framework::JSON::Validator::ObjectValidationBuilder<
            Framework::JSON::Validator::JSONValidator>* builder) const override;
    Framework::Text getTypeToken() const override;
};