#pragma once

#include "BasicBlocks.h"
#include "Block.h"
#include "BlockType.h"

class BasicLightSourceBlockType;
class LightSourceItemType;

class LightSourceItem : public BasicBlockItem
{
protected:
    int color;

public:
    LightSourceItem(int itemType, int blockType, Framework::Text name);
    virtual bool canBeStackedWith(const Item* zItem) const override;

    friend LightSourceItemType;
    friend BasicLightSourceBlockType;
};

class LightSourceItemType : public BasicBlockItemType
{
private:
    int color;

public:
    LightSourceItemType(Framework::Text name,
        ModelInfo* model,
        Framework::Text blockTypeName,
        int maxStackSize);

protected:
    virtual void loadSuperItem(
        Item* zItem, Framework::StreamReader* zReader) const override;
    virtual void saveSuperItem(
        const Item* zItem, Framework::StreamWriter* zWriter) const override;
    virtual Item* createItem() const override;

public:
    LightSourceItemType* setColor(int color);
};

class BasicLightSource : public Block
{
public:
    BasicLightSource(int typeId, Framework::Vec3<int> pos, int dimensionId);

protected:
    void setLight(int light);

public:
    virtual bool onTick(
        TickQueue* zQueue, int numTicks, bool& blocked) override;
    virtual void onPostTick() override;

    friend BasicLightSourceBlockType;
};

class BasicLightSourceBlockType : public BlockType
{
private:
    Framework::Text itemTypeName;
    int itemTypeId;
    bool transparent;
    bool passable;
    bool interactable;
    int lightColor;

public:
    BasicLightSourceBlockType(Framework::Text itemTypeName,
        ModelInfo* model,
        Framework::Text name,
        float hardness,
        int mapColor,
        int lightColor,
        Framework::RCArray<Framework::Text> groupNames);

protected:
    virtual void createSuperBlock(Block* zBlock, Item* zItem) const override;
    virtual void loadSuperBlock(Block* zBlock,
        Framework::StreamReader* zReader,
        int dimensionId) const override;
    virtual void saveSuperBlock(
        Block* zBlock, Framework::StreamWriter* zWriter) const override;

public:
    virtual bool initialize(Game* zGame) override;
    virtual Block* createBlock(
        Framework::Vec3<int> position, int dimensionId) const override;
    virtual Item* createItem() const override;

    int getLightColor() const;
    Framework::Text getItemTypeName() const;
    virtual ItemType* createItemType() const override;
};

class BasicLightSourceBlockTypeFactory
    : public SubTypeFactory<BlockType, BasicLightSourceBlockType>
{
public:
    BasicLightSourceBlockTypeFactory();
    BasicLightSourceBlockType* fromJson(
        Framework::JSON::JSONObject* zJson) const override;
    Framework::JSON::JSONObject* toJson(
        BasicLightSourceBlockType* zObject) const override;
    Framework::JSON::Validator::JSONValidator* getValidator(
        Framework::JSON::Validator::ObjectValidationBuilder<
            Framework::JSON::Validator::JSONValidator>* builder) const override;
    Framework::Text getTypeToken() const override;
};